import unittest
#from faker import Faker
class Faker:
	pass
fake = Faker()

def generate_person():
    """Generates a random person's information."""
    return {
        "name": fake.name(),
        "address": fake.address(),
        "email": fake.email(),
        "phone": fake.phone_number(),
        "dob": fake.date_of_birth(minimum_age=18, maximum_age=90).isoformat(),
        "ssn": fake.ssn()
    }
class TestGenerator(unittest.TestCase):
    def test_generate_person(self):
        person = generate_person()
        self.assertIn("name", person)
        self.assertIn("email", person)
        self.assertTrue(person["name"])
        self.assertTrue(person["email"])

if __name__ == "__main__":
    unittest.main()



from egyptian.__init__ import *
class IDParser:

	@staticmethod
	def validate_id(id_number):
		if not id_number.isdigit() or len(id_number) != 14:
			raise ValueError("ID Number Must Be 14 Digits Long.")

	@staticmethod
	def extract_birth_date(id_number):
		century_code = id_number[0]
		year = id_number[1:3]
		month = id_number[3:5]
		day = id_number[5:7]

		if century_code == '2':
			birth_year = '19' + year
		elif century_code == '3':
			birth_year = '20' + year
		else:
			raise ValueError("Invalid century code in ID")

		try:
			birth_date = datetime.datetime.strptime(f"{birth_year}-{month}-{day}", "%Y-%m-%d").date()
		except ValueError:
			raise ValueError("Invalid date in ID.")

		return birth_date

	@staticmethod
	def extract_governorate(id_number):
		governorate_code = id_number[7:9]
		return IDParser.governorate_map.get(governorate_code, "Unknown Governorate")

	@staticmethod
	def extract_gender(id_number):
		serial_number = int(id_number[9:13])
		return 'Male' if serial_number % 2 != 0 else 'Female'

	@staticmethod
	def calculate_age_components(birth_date):
		today = datetime.date.today()
		years = today.year - birth_date.year
		months = today.month - birth_date.month
		days = today.day - birth_date.day

		if days < 0:
			months -= 1
			days += (birth_date + datetime.timedelta(days=30)).day
		if months < 0:
			months += 12
			years -= 1

		return years, months, days



from egyptian.generators import *

class Person:
	def __init__(self, language: str=None,gender: str=None, name_length: int=None,age: int=None,name: str=None,city: str=None,job_title:str=None,phone:str=None,
	             user_name:str=None,email:str=None,national_id:str=None,
	             phone_carrier:str=None,phone_start_with:str=None, user_name_lenght:int=None,birth_day=None,birth_month=None,birth_year=None,email_ends_with=None):
		# Store only user-provided arguments (ignore None values)
		self.user_provided_args = {k: v for k, v in locals().items() if k != 'self' and v is not None}
		if not name_length:
			name_length = 2
		if not user_name_lenght:
			user_name_lenght = 15
		if not language:
			self.language = choice(languages)
		else:
			language_check(language)
			self.language = language
		if not gender:
			if self.language.lower().startswith('ar'):
				self.gender = choice(arabic_genders)
			else:
				self.gender = choice(english_genders)
		else:
			gender_check(gender)
			self.gender = gender
			if self.language.lower().startswith('ar'):
				if gender in male_genders:
					self.gender = 'ذكر'
				else:
					self.gender = 'أنثي'
			else:
				if gender in male_genders:
					self.gender = 'male'
				else:
					self.gender = 'female'
		self.name_length = name_length
		if birth_year and age:
			raise Exception("You can't pass both birth_year and age")
		if not age:
			age = randint(0, 100)
			temp_age = age
			if self.language.lower().startswith('ar'):
				age = convert_to_arabic_numerals(str(age))
		else:
			temp_age = age
		self.age = age
		if not birth_year:
			birth_year = datetime.now().year - temp_age
			if self.language.lower().startswith('ar'):
				birth_year = convert_to_arabic_numerals(str(birth_year))
		self.birth_year = birth_year
		if not birth_month:
			birth_month = randint(1, 12)
			if self.language.lower().startswith('ar'):
				birth_month = convert_to_arabic_numerals(str(birth_month))
		self.birth_month = birth_month
		if not birth_day:
			birth_day = randint(1, 30)
			if self.language.lower().startswith('ar'):
				birth_day = convert_to_arabic_numerals(str(birth_day))
		self.birth_day = birth_day
		if not name:
			name = generate_name(language=self.language,gender=self.gender,name_length=self.name_length)
		self.name = name
		if not city:
			city = generate_city(language=self.language)
		self.city = city
		if not job_title:
			job_title = generate_job_title(language=self.language,gender=self.gender)
		self.job_title = job_title
		if not phone_carrier:
			self.phone_carrier = choice(phone_carriers)
		else:
			phone_carrier_check(phone_carrier)
			self.phone_carrier = phone_carrier
		if not phone_start_with:
			if self.language.lower().startswith('ar'):
				self.phone_start_with = choice(arabic_phone_start_withs)
			else:
				self.phone_start_with = choice(english_phone_start_withs)
		else:
			phone_start_with_check(phone_start_with)
			self.phone_start_with = phone_start_with
		if not phone:
			phone = generate_phone(language=self.language,phone_carrier=self.phone_carrier,phone_start_with=self.phone_start_with)
		self.phone = phone
		self.user_name_length = user_name_lenght
		if not user_name:
			user_name = generate_user_name(language=self.language,user_name_length=self.user_name_length,name=self.name)
		self.user_name = user_name
		if email and email_ends_with:
			raise Exception("You can't pass both email and email_ends_with")
		if not email_ends_with:
			self.email_ends_with = choice(email_ends_withs)
		else:
			email_ends_with_check(email_ends_with)
			self.email_ends_with = email_ends_with
		if not email:
			email = f"{self.user_name}@{self.email_ends_with}"
		self.email = email
		if not national_id:
			national_id = generate_national_id(language=self.language,birth_day=self.birth_day,birth_month=self.birth_month,birth_year=self.birth_year,gender=self.gender)
		self.national_id = national_id
	def update_name(self,language=None,name_length=None,gender=None):
		data = {"language":self.language,"gender":self.gender,"name_length":self.name_length}
		if language:
			data["language"] = language
		if name_length:
			data["name_length"] = name_length
		if gender:
			data["gender"] = gender
		name = generate_name(**data)
		self.name = name
		return self.name

	def update_user_name(self,language=None,user_name_length=None,name=None):
		data = {"language": self.language, "user_name_length": self.name_length,"name":self.name}
		if language:
			data["language"] = language
		if user_name_length:
			data["user_name_length"] = user_name_length
		if name:
			data["name"] = name
		user_name = generate_user_name(**data)
		self.user_name = user_name
		return self.user_name
	def update_job_title(self,language=None,gender=None):
		data = {"language": self.language, "gender": self.gender}
		if language:
			data["language"] = language
		if gender:
			data["gender"] = gender
		job_title = generate_job_title(**data)
		self.job_title = job_title
		return self.job_title

	def update_email(self,language=None,user_name_length=None,name=None):
		data = {"language": self.language, "user_name_length": self.user_name_length,"name":self.name}
		if language:
			data["language"] = language
		if name:
			data["name"] = name
		if user_name_length:
			data["user_name_length"] = user_name_length
		user_name = generate_user_name(**data)
		self.email = f"{user_name}@{self.email_ends_with}"
		return self.email
	def update_national_id(self,language=None,birth_day=None,birth_month=None,birth_year=None,gender=None):
		data = {"language": self.language, "gender": self.gender,"birth_day": self.birth_day,"birth_month": self.birth_month,"birth_year": self.birth_year}
		if language:
			data["language"] = language
		if birth_day:
			data["birth_day"] = birth_day
		if birth_month:
			data["birth_month"] = birth_month
		if birth_year:
			data["birth_year"] = birth_year
		if gender:
			data["gender"] = gender
		national_id = generate_national_id(**data)
		self.national_id = national_id
		return self.national_id
	def update_phone(self,language=None,phone_carrier=None,phone_start_with=None):
		data = {"language": self.language, "gender": self.gender, "name_length": self.name_length}
		if language:
			data["language"] = language
		if phone_carrier:
			data["phone_carrier"] = phone_carrier
		if phone_start_with:
			data["phone_start_with"] = phone_start_with
		phone = generate_phone(**data)
		self.phone = phone
		return self.phone
	def translate_name(self):
		translated_name = ""
		for name in self.name.split(" "):
			name = translate_name(name)
			translated_name += f"{name} "
		self.name = translated_name
		return self.name
	def translate_job_title(self):
		translated_job_title = translate_job_title(self.job_title)
		self.job_title = translated_job_title
		return self.job_title
	def translate_gender(self):
		translated_gender = translate_gender(self.gender)
		self.gender = translated_gender
		return self.gender
	def translate_city(self):
		translated_city = translate_city(self.city)
		self.city = translated_city
		return self.city
	def translate_phone(self):
		translated_phone = translate_number(self.phone)
		self.phone = translated_phone
		return self.phone
	def translate_national_id(self):
		translated_national_id = translate_number(self.national_id)
		self.national_id = translated_national_id
		return self.national_id
	def translate_birth_year(self):
		translated_birth_year = translate_number(self.birth_year)
		self.birth_year = translated_birth_year
		return self.birth_year
	def translate_birth_month(self):
		translated_birth_month = translate_number(self.birth_month)
		self.birth_month = translated_birth_month
		return self.birth_month
	def translate_birth_day(self):
		translated_birth_day = translate_number(self.birth_day)
		self.birth_day = translated_birth_day
		return self.birth_day


	def get_all(self,number_of_persons:int=1):
		if number_of_persons == 1:
			result = deepcopy(self.__dict__)
			filter_result(result)
			return {**result}
		else:
			persons = []
			data = self.user_provided_args
			for i in range(number_of_persons):
				person = Person(**data)
				result = deepcopy(person.__dict__)
				filter_result(result)
				persons.append(result)
			return persons




male_names = {
    "Ahmed": "أحمد", "Mohamed": "محمد", "Ali": "علي", "Omar": "عمر", "Youssef": "يوسف", "Ibrahim": "إبراهيم", "Hassan": "حسن", "Hussein": "حسين", "Khaled": "خالد", "Mahmoud": "محمود",
    "Mostafa": "مصطفى", "Amr": "عمرو", "Tamer": "تامر", "Karim": "كريم", "Samir": "سمير", "Sherif": "شريف", "Gamal": "جمال", "Ehab": "إيهاب", "Ashraf": "أشرف", "Adel": "عادل",
    "Sami": "سامي", "Tariq": "طارق", "Waleed": "وليد", "Hany": "هاني", "Saad": "سعد", "Fady": "فادي", "Bassem": "باسم", "Ezzat": "عزت", "Ramy": "رامي", "Nader": "نادر",
    "Marwan": "مروان", "Ayman": "أيمن", "Reda": "رضا", "Wael": "وائل", "Hesham": "هشام", "Sayed": "سيد", "Shady": "شادي", "Ziad": "زياد", "Magdy": "مجدي", "Lotfy": "لطفي",
    "Medhat": "مدحت", "Kareem": "كريم", "Yasser": "ياسر", "Fouad": "فؤاد", "Anwar": "أنور", "Gaber": "جابر", "Shokry": "شكري", "Salah": "صلاح", "Mostafa": "مصطفى", "Bahaa": "بهاء",
    "Ismail": "إسماعيل", "Hamdy": "حمدي", "Shawky": "شوقي", "Emad": "عماد", "Osama": "أسامة", "Hatem": "حاتم", "Nabil": "نبيل", "Lotfi": "لطفي", "Alaa": "علاء", "Sameh": "سامح",
    "Fathi": "فتحي", "Farid": "فريد", "Bassel": "باسل", "Ehab": "إيهاب", "Sabry": "صبري", "Sayed": "سيد", "Shereef": "شريف", "Essam": "عصام", "Sherbini": "الشربيني", "Ragab": "رجب",
    "Hany": "هاني", "Bishoy": "بيشوي", "Mina": "مينا", "Victor": "فيكتور", "Adham": "أدهم", "Tawfiq": "توفيق", "Hamza": "حمزة", "Saif": "سيف", "Fares": "فارس", "Moataz": "معتز",
    "Wahid": "وحيد", "Ihab": "إيهاب", "Nashaat": "نشأت", "Galal": "جلال", "Hady": "هادي", "Salem": "سالم", "Sherif": "شريف", "Emam": "إمام", "Refaat": "رفعت", "Bakr": "بكر",
    "Mamdouh": "ممدوح", "Montaser": "منتصر", "Abdelrahman": "عبد الرحمن", "Saber": "صابر", "Makram": "مكرم", "Gerges": "جرجس", "Eid": "عيد", "Fawzy": "فوزي", "Anas": "أنس", "Raouf": "رؤوف"
}
female_names = {
    "Amina": "أمينة", "Fatma": "فاطمة", "Nour": "نور", "Mona": "منى", "Dina": "دينا", "Heba": "هبة", "Samar": "سمر", "Marwa": "مروة", "Rania": "رانيا", "Yasmin": "ياسمين",
    "Nadia": "نادية", "Layla": "ليلى", "Hanan": "حنان", "Nihal": "نهال", "Nagwa": "نجوى", "Zahraa": "زهراء", "Sarah": "سارة", "Maha": "مها", "Noha": "نهى", "Doaa": "دعاء",
    "Abeer": "عبير", "Shimaa": "شيماء", "Eman": "إيمان", "Riham": "ريهام", "Ghada": "غادة", "Mervat": "مرفت", "Walaa": "ولاء", "Sally": "سالي", "Hadeer": "هدير", "Mai": "مي",
    "Shaimaa": "شيماء", "Esraa": "إسراء", "Amal": "أمل", "Lobna": "لبنى", "Nourhan": "نورهان", "Dalia": "داليا", "Hind": "هند", "Karima": "كريمة", "Rasha": "رشا", "Ruqayya": "رقية",
    "Nahla": "نهلة", "Bassma": "بسمة", "Maram": "مرام", "Rehab": "رحاب", "Manar": "منار", "Huda": "هدى", "Nermine": "نرمين", "Hagar": "هاجر", "Sahar": "سحر", "Ferial": "فريال",
    "Lubna": "لبنى", "Basma": "بسمة", "Mariam": "مريم", "Randa": "راندا", "Salma": "سلمى", "Tahani": "تهاني", "Zeinab": "زينب", "Asmaa": "أسماء", "Gihan": "جيهان", "Samia": "سامية",
    "Wafaa": "وفاء", "Sondos": "سندس", "Hala": "هالة", "Sabah": "صباح", "Yosra": "يسرا", "Dorra": "درة", "Rim": "ريم", "Rania": "رانيا", "Elham": "إلهام", "Hanadi": "هنادي",
    "Rawan": "روان", "Lamia": "لمياء", "Reem": "ريم", "Rokaya": "رقية", "Suhair": "سهير", "Sherine": "شيرين", "Zina": "زينة", "Fawzia": "فوزية", "Sahar": "سحر", "Lina": "لينا",
    "Ahlam": "أحلام", "Mariana": "ماريانا", "Maysa": "ميساء", "Fadia": "فادية", "Ghada": "غادة", "Najla": "نجلاء", "Iman": "إيمان", "Salsabil": "سلسبيل", "Afaf": "عفاف", "Rimonda": "ريموندا",
    "Malak": "ملك", "Esraa": "إسراء", "Jana": "جنى", "Laila": "ليلى", "Rasha": "رشا", "Mirna": "ميرنا", "Dalida": "داليدا", "Khadija": "خديجة", "Sondos": "سندس", "Yasmina": "ياسمينة"
}
male_job_titles = {
    "Engineer": "مهندس", "Doctor": "طبيب", "Teacher": "معلم", "Accountant": "محاسب", "Lawyer": "محام", "Architect": "مهندس معماري", "Pharmacist": "صيدلي", "Journalist": "صحفي", "Dentist": "طبيب أسنان", "Electrician": "كهربائي",
    "Plumber": "سباك", "Carpenter": "نجار", "Mechanic": "ميكانيكي", "Chef": "طاهٍ", "Driver": "سائق", "Pilot": "طيار", "Soldier": "جندي", "Police Officer": "ضابط شرطة", "Firefighter": "رجل إطفاء", "Scientist": "عالم",
    "Professor": "أستاذ جامعي", "Researcher": "باحث", "Economist": "اقتصادي", "Psychologist": "طبيب نفسي", "Therapist": "معالج", "Veterinarian": "طبيب بيطري", "Artist": "فنان", "Musician": "موسيقي", "Actor": "ممثل", "Director": "مخرج",
    "Photographer": "مصور", "Designer": "مصمم", "Writer": "كاتب", "Editor": "محرر", "Translator": "مترجم", "IT Specialist": "أخصائي تكنولوجيا المعلومات", "Software Developer": "مطور برمجيات", "Web Developer": "مطوّر ويب", "Graphic Designer": "مصمم جرافيك", "Data Analyst": "محلل بيانات",
    "Statistician": "إحصائي", "Cybersecurity Analyst": "محلل أمن سيبراني", "Network Engineer": "مهندس شبكات", "System Administrator": "مسؤول أنظمة", "Marketing Specialist": "أخصائي تسويق", "Sales Manager": "مدير مبيعات", "HR Specialist": "أخصائي موارد بشرية", "Consultant": "مستشار", "Banker": "موظف بنك", "Financial Analyst": "محلل مالي",
    "Investment Manager": "مدير استثمار", "Stockbroker": "سمسار بورصة", "Insurance Agent": "وكيل تأمين", "Auditor": "مدقق حسابات", "Public Relations Specialist": "أخصائي علاقات عامة", "Event Planner": "منظم فعاليات", "Tour Guide": "مرشد سياحي", "Hotel Manager": "مدير فندق", "Waiter": "نادل", "Baker": "خباز",
    "Butcher": "جزار", "Fisherman": "صياد", "Farmer": "مزارع", "Construction Worker": "عامل بناء", "Real Estate Agent": "وكيل عقارات", "Security Guard": "حارس أمن", "Janitor": "عامل نظافة", "Tailor": "خياط", "Shoemaker": "إسكافي", "Barber": "حلاق",
    "Flight Attendant": "مضيف طيران", "Ship Captain": "قبطان سفينة", "Athlete": "رياضي", "Coach": "مدرب", "Referee": "حكم", "Gym Trainer": "مدرب رياضي", "Priest": "قسيس", "Imam": "إمام", "Monk": "راهب", "Social Worker": "أخصائي اجتماعي",
    "Volunteer": "متطوع", "Judge": "قاضٍ", "Politician": "سياسي", "Ambassador": "سفير", "Diplomat": "دبلوماسي", "Government Officer": "موظف حكومي", "Entrepreneur": "رائد أعمال", "Business Owner": "صاحب عمل", "Manager": "مدير", "Supervisor": "مشرف",
    "Clerk": "كاتب", "Receptionist": "موظف استقبال", "Courier": "ساعٍ", "Delivery Driver": "سائق توصيل", "Warehouse Worker": "عامل مستودع", "Logistics Coordinator": "منسق لوجستي"
}
female_job_titles = {
    "Engineer": "مهندسة", "Doctor": "طبيبة", "Teacher": "معلمة", "Accountant": "محاسبة", "Lawyer": "محامية", "Architect": "مهندسة معمارية", "Pharmacist": "صيدلانية", "Journalist": "صحفية", "Dentist": "طبيبة أسنان", "Electrician": "كهربائية",
    "Plumber": "سباكة", "Carpenter": "نجارة", "Mechanic": "ميكانيكية", "Chef": "طاهية", "Driver": "سائقة", "Pilot": "طيارة", "Soldier": "جندية", "Police Officer": "ضابطة شرطة", "Firefighter": "إطفائية", "Scientist": "عالمة",
    "Professor": "أستاذة جامعية", "Researcher": "باحثة", "Economist": "اقتصادية", "Psychologist": "طبيبة نفسية", "Therapist": "معالجة", "Veterinarian": "طبيبة بيطرية", "Artist": "فنانة", "Musician": "موسيقية", "Actor": "ممثلة", "Director": "مخرجة",
    "Photographer": "مُصورة", "Designer": "مصممة", "Writer": "كاتبة", "Editor": "محررة", "Translator": "مترجمة", "IT Specialist": "أخصائية تكنولوجيا المعلومات", "Software Developer": "مطورة برمجيات", "Web Developer": "مطوّرة ويب", "Graphic Designer": "مصممة جرافيك", "Data Analyst": "محللة بيانات",
    "Statistician": "إحصائية", "Cybersecurity Analyst": "محللة أمن سيبراني", "Network Engineer": "مهندسة شبكات", "System Administrator": "مسؤولة أنظمة", "Marketing Specialist": "أخصائية تسويق", "Sales Manager": "مديرة مبيعات", "HR Specialist": "أخصائية موارد بشرية", "Consultant": "مستشارة", "Banker": "موظفة بنك", "Financial Analyst": "محللة مالية",
    "Investment Manager": "مديرة استثمار", "Stockbroker": "سمسارة بورصة", "Insurance Agent": "وكيلة تأمين", "Auditor": "مدققة حسابات", "Public Relations Specialist": "أخصائية علاقات عامة", "Event Planner": "منظمة فعاليات", "Tour Guide": "مرشدة سياحية", "Hotel Manager": "مديرة فندق", "Waiter": "نادلة", "Baker": "خبازة",
    "Butcher": "جزارة", "Fisherman": "صيادة", "Farmer": "مزارعة", "Construction Worker": "عاملة بناء", "Real Estate Agent": "وكيلة عقارات", "Security Guard": "حارسة أمن", "Janitor": "عاملة نظافة", "Tailor": "خياطة", "Shoemaker": "إسكافية", "Barber": "حلاقة",
    "Flight Attendant": "مضيفة طيران", "Ship Captain": "قبطانة سفينة", "Athlete": "رياضية", "Coach": "مدربة", "Referee": "حكمة", "Gym Trainer": "مدربة رياضية", "Priest": "قسيسة", "Imam": "إمامة", "Monk": "راهبة", "Social Worker": "أخصائية اجتماعية",
    "Volunteer": "متطوعة", "Judge": "قاضية", "Politician": "سياسية", "Ambassador": "سفيرة", "Diplomat": "دبلوماسية", "Government Officer": "موظفة حكومية", "Entrepreneur": "رائدة أعمال", "Business Owner": "صاحبة عمل", "Manager": "مديرة", "Supervisor": "مشرفة",
    "Clerk": "كاتبة", "Receptionist": "موظفة استقبال", "Courier": "ساعية", "Delivery Driver": "سائقة توصيل", "Warehouse Worker": "عاملة مستودع", "Logistics Coordinator": "منسقة لوجستية"
}
egyptian_cities = {
    "Cairo": "القاهرة", "Alexandria": "الإسكندرية", "Giza": "الجيزة", "Shubra El Kheima": "شبرا الخيمة", "Port Said": "بورسعيد", "Suez": "السويس", "Luxor": "الأقصر", "Asyut": "أسيوط", "Ismailia": "الإسماعيلية", "Faiyum": "الفيوم",
    "Zagazig": "الزقازيق", "Damietta": "دمياط", "Aswan": "أسوان", "Mansoura": "المنصورة", "Tanta": "طنطا", "Minya": "المنيا", "Beni Suef": "بني سويف", "Hurghada": "الغردقة", "Qena": "قنا", "Sohag": "سوهاج",
    "Sharm El Sheikh": "شرم الشيخ", "Damanhur": "دمنهور", "Banha": "بنها", "Kafr El Sheikh": "كفر الشيخ", "Arish": "العريش", "Mallawi": "ملوي", "Bilharz": "بلحارس", "Marsa Matruh": "مرسى مطروح", "New Cairo": "القاهرة الجديدة", "Obour": "العبور",
    "6th of October City": "مدينة 6 أكتوبر", "Sheikh Zayed City": "مدينة الشيخ زايد", "Madinaty": "مدينتي", "Sadat City": "مدينة السادات", "10th of Ramadan City": "مدينة العاشر من رمضان", "Helwan": "حلوان", "Shibin El Kom": "شبين الكوم", "Desouk": "دسوق", "Qalyub": "قليوب", "Al Mahallah Al Kubra": "المحلة الكبرى",
    "Mit Ghamr": "ميت غمر", "Kafr Saad": "كفر سعد", "Toukh": "طوخ", "Samalut": "سمالوط", "Beni Mazar": "بني مزار", "Abnub": "أبنوب", "El Badari": "البداري", "El Quseir": "القصير", "Ras Gharib": "رأس غارب", "Safaga": "سفاجا",
    "El Alamein": "العلمين", "Matay": "مطاي", "Kom Ombo": "كوم أمبو", "Edku": "إدكو", "Rosetta": "رشيد", "Abu Simbel": "أبو سمبل", "Akhmim": "أخميم", "Farshut": "فرشوط", "Edfu": "إدفو", "Girga": "جرجا"
}
arabic_numerals = ['٠', '١', '٢', '٣', '٤', '٥', '٦', '٧', '٨', '٩']
english_numerals = ['0', '1', '2', '3', '4', '5', '6', '7', '8', '9']
email_ends_withs = ['gmail.com', 'yahoo.com', 'outlook.com', 'hotmail.com', 'aol.com', 'icloud.com', 'protonmail.com', 'zoho.com', 'gmx.com', 'yandex.com']
arabic_genders = ['ذكر', 'أنثي']
english_genders = ['male', 'female']
genders = [*arabic_genders, *english_genders]
male_genders = ['ذكر', 'male']
female_genders = ['أنثي', 'female']
languages = ['ar', 'en',"arabic","english"]
arabic_phone_start_withs = [ '٢',"+٢",""]
english_phone_start_withs = ['2', '+2',""]
phone_start_withs = [*english_phone_start_withs,*arabic_phone_start_withs]
phone_carriers = ['orange', 'we', 'vodafone',"etisalat"]
governorate_map = {
		'01': 'Cairo',
		'02': 'Alexandria',
		'03': 'Port Said',
		'04': 'Suez',
		'11': 'Damietta',
		'12': 'Dakahlia',
		'13': 'Sharqia',
		'14': 'Qalyubia',
		'15': 'Kafr El Sheikh',
		'16': 'Gharbia',
		'17': 'Monufia',
		'18': 'Beheira',
		'19': 'Ismailia',
		'21': 'Giza',
		'22': 'Beni Suef',
		'23': 'Faiyum',
		'24': 'Minya',
		'25': 'Asyut',
		'26': 'Sohag',
		'27': 'Qena',
		'28': 'Aswan',
		'29': 'Luxor',
		'31': 'Red Sea',
		'32': 'New Valley',
		'33': 'Matrouh',
		'34': 'North Sinai',
		'35': 'South Sinai',
	}


from egyptian.db import *
from egyptian.__init__ import *
def convert_to_arabic_numerals(english_number: str) -> str:
	translation_table = str.maketrans("".join(english_numerals), "".join(arabic_numerals))
	return english_number.translate(translation_table)
def convert_to_english_numerals(arabic_number: str) -> str:
	translation_table = str.maketrans("".join(arabic_numerals),"".join(english_numerals))
	return arabic_number.translate(translation_table)

def filter_result(result:dict):
	for field in ['user_provided_args', 'user_name_length',"name_length","email_ends_with","phone_carrier","phone_start_with","language"]:
		result.pop(field)
	return result

def gender_check(gender):
	assert gender.lower() in genders,"gender must be one of {}".format(genders)

def language_check(language):
	assert language.lower() in languages,"language must be one of {}".format(languages)

def phone_start_with_check(phone_start_with):
	assert phone_start_with.lower() in phone_start_withs,"phone_start_with must be one of {}".format(phone_start_withs)

def phone_carrier_check(phone_carrier):
	assert phone_carrier.lower() in phone_carriers,"phone_carrier must be one of {}".format(phone_carriers)


def email_ends_with_check(email_ends_with):
	assert email_ends_with.lower() in email_ends_withs,"email_ends_with must be one of {}".format(email_ends_withs)

def generate_random_value(values:list,lenght:int=1):
	return "".join([choice(values) for i in range(lenght)])
def generate_name(language:str,gender,name_length):
	language_check(language)
	gender_check(gender)
	if language.lower().startswith('ar'):
		male_names_list = list(male_names.values())
		female_names_list = list(female_names.values())
	else:
		male_names_list = list(male_names.keys())
		female_names_list = list(female_names.keys())
	if gender in male_genders:
		name = choice(male_names_list)
	else:
		name = choice(female_names_list)
	for i in range(name_length-1):
		name += f" {choice(male_names_list)}"
	return name
def generate_city(language:str):
	language_check(language)
	if language.lower().startswith('ar'):
		return choice(list(egyptian_cities.values()))
	else:
		return choice(list(egyptian_cities.keys()))
def generate_job_title(language:str,gender):
	language_check(language)
	gender_check(gender)
	if language.lower().startswith('ar'):
		male_job_tiles = list(male_job_titles.values())
		female_job_tiles = list(female_job_titles.values())

	else:
		male_job_tiles = list(male_job_titles.keys())
		female_job_tiles = list(female_job_titles.keys())
	if gender in male_genders:
		job_title = choice(male_job_tiles)
	else:
		job_title = choice(female_job_tiles)
	return job_title
def generate_phone(language:str,phone_carrier:str,phone_start_with):
	language_check(language)
	from .Randomizer import phone_carriers
	if language.lower().startswith('ar'):
		numerals = arabic_numerals
	else:
		numerals = english_numerals
	if phone_carrier.lower() == 'vodafone':
		phone_perfix = f"{numerals[0]}{numerals[1]}{numerals[0]}"
	elif phone_carrier.lower() == 'etisalat':
		phone_perfix = f"{numerals[0]}{numerals[1]}{numerals[1]}"
	elif phone_carrier.lower() == 'we':
		phone_perfix = f"{numerals[0]}{numerals[1]}{numerals[5]}"
	elif phone_carrier.lower() == 'orange':
		phone_perfix = f"{numerals[0]}{numerals[1]}{numerals[2]}"
	else:
		raise Exception("phone_carrier must be one of {}".format(phone_carriers))
	return f"{phone_start_with}{phone_perfix}" + f"{generate_random_value(numerals,8)}"
def generate_user_name(language:str,user_name_length,name:str):
	language_check(language)
	if language.lower().startswith('ar'):
		numerals = arabic_numerals
	else:
		numerals = english_numerals
	user_name = name.replace(" ","")
	if len(user_name) > user_name_length:
		user_name = user_name[:user_name_length]
	else:
		user_name = f"{user_name}" + generate_random_value(numerals,(user_name_length-len(user_name)))
	return user_name
def generate_national_id(language:str,gender:str,birth_day:int,birth_month:int,birth_year:int):
	language_check(language)
	gender_check(gender)
	if language.lower().startswith('ar'):
		numerals = arabic_numerals
	else:
		numerals = english_numerals
	odd_numerals = [numerals[1], numerals[3], numerals[5], numerals[7], numerals[9]]
	even_numerals = [numerals[0], numerals[2], numerals[4], numerals[6], numerals[8]]
	birth_century = str(birth_year)[:2]
	if language.lower().startswith('ar'):
		birth_century = convert_to_arabic_numerals(birth_century)
	else:
		birth_century = convert_to_english_numerals(birth_century)
	if birth_century == f"{numerals[1]}{numerals[8]}":
		birth_century_value = numerals[1]
	elif birth_century == f"{numerals[1]}{numerals[9]}":
		birth_century_value = numerals[2]
	elif birth_century == f"{numerals[2]}{numerals[0]}":
		birth_century_value = numerals[3]
	else:
		raise Exception("birth_century must be between 18 and 20")
	birth_year = str(birth_year)[2:]
	birth_day = str(birth_day).zfill(2)
	birth_month = str(birth_month).zfill(2)
	if language.lower().startswith('ar'):
		birth_day = convert_to_arabic_numerals(birth_day)
		birth_month = convert_to_arabic_numerals(birth_month)
		birth_year = convert_to_arabic_numerals(birth_year)
	else:
		birth_day = convert_to_english_numerals(birth_day)
		birth_month = convert_to_english_numerals(birth_month)
		birth_year = convert_to_english_numerals(birth_year)
	if gender in male_genders:
		gender_value = f"{choice(odd_numerals)}"
	else:
		gender_value = f"{choice(even_numerals)}"

	return f"{birth_century_value}{birth_year}{birth_month}{birth_day}"+ f"{generate_random_value(numerals,5)}"+f"{gender_value}"
def translate_name(name):
	if name in male_names.keys():
		return male_names[name]
	elif name in female_names.keys():
		return female_names[name]
	elif name in male_names.values():
		return next((k for k, v in male_names.items() if v == name), None)
	elif name in female_names.values():
		return next((k for k, v in female_names.items() if v == name), None)
	else:
		return name
def translate_job_title(job_title):
	if job_title in male_job_titles.keys():
		return male_job_titles[job_title]
	elif job_title in female_job_titles.keys():
		return female_job_titles[job_title]
	elif job_title in male_job_titles.values():
		return next((k for k, v in male_job_titles.items() if v == job_title), None)
	elif job_title in female_job_titles.values():
		return next((k for k, v in female_job_titles.items() if v == job_title), None)
	else:
		return job_title
def translate_city(city):
	if city in egyptian_cities.keys():
		return egyptian_cities[city]
	elif city in egyptian_cities.values():
		return next((k for k, v in egyptian_cities.items() if v == city), None)
	else:
		return city
def translate_number(number):
	number = str(number)
	translated_number = ""
	for n in number:
		if n in arabic_numerals:
			n = english_numerals[arabic_numerals.index(n)]
		elif n in english_numerals:
			n = arabic_numerals[english_numerals.index(n)]
		translated_number += n
	return translated_number
def translate_gender(gender):
	if gender in arabic_genders:
		return english_genders[arabic_genders.index(gender)]
	elif gender in english_genders:
		return arabic_genders[english_genders.index(gender)]
	else:
		return gender
